from datetime import datetime, timedelta
from statistics import mean
from typing import Iterable

# 1. Proposer une classe Annuaire qui comprends des champs nom, prenom et numéro et historique.

class Annuaire:
    def __init__(self, nom: str, prenom: str, numero:str):
        self._nom = nom
        self._prenom = prenom
        self._numero = numero 
        self._historique = []

    @property
    def nom(self) -> str:
        return self._nom

    @property
    def prenom(self) -> str:
        return self._prenom

    @property
    def numero(self) -> str:
        return self._numero

    @property
    def historique(self) -> list[str]:
        return self._historique

    def nouvel_appel(self, date, duree):
        self._historique.append(Appel(date, duree))

    def __repr__(self):
        return f"Annuaire({self.prenom}, {self.nom})"

    @property
    def dernier_appel(self) -> "Appel":
        return max(self.historique, key=lambda e:e.date)

    @property
    def duree_moyenne(self) -> float:
        return mean(map(lambda e:e.duree, self.historique))

    @property
    def nombre_appel(self) -> int:
        return len(self.historique)


    def __lt__(self, other) -> bool:
        if self.nombre_appel < other.nombre_appel:
            return True
        if self.duree_moyenne < other.duree_moyenne:
            return True
        if self.dernier_appel.date < other.dernier_appel.date:
            return True
        return False

    def __le__(self, other) -> bool:
        return (not self < other) and (not other < self)
class Appel:
    def __init__(self, date: datetime, duree: int):
        self._date = date
        self._duree = duree

    @property
    def date(self) -> datetime:
        return self._date

    @property
    def duree(self) -> int:
        return self._duree

    @property
    def date_fin(self) -> datetime:
        return self.date + timedelta(seconds=self.duree)

    def __repr__(self):
        return f"Appel({self.date}, {self.duree})"

if __name__ == "__main__":
    A = Annuaire("Charles", "Paperman", "06262626")
    print(A.historique) # retourne []
    
    try:
        A.nom = "pouet"
    except AttributeError:
        print("C'est interdit!!")

def plus_important(list_annuaire: Iterable[Annuaire]) -> tuple[Annuaire,...]:
    return tuple(sorted(list_annuaire)[-5:])
